"""Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


from collections import OrderedDict


class Record(OrderedDict):
    """Base class for log record."""

    TIME = "time"
    SCALE = "scale"

    def __init__(self, time, scale):
        """Default constructor."""
        OrderedDict.__init__(self)
        self[self.TIME] = int(time)
        self[self.SCALE] = scale

    def get_time(self):
        """Returns the timestamp (decimal value)."""
        return self[self.TIME]

    def get_scale(self):
        """Returns the unit that indicates the timestamp is not related to real time, but an increasing count."""
        return self[self.SCALE]

    def to_string(self):
        """Returns the string representation of this record, None fields are removed."""
        data = []
        for value in self.values():
            if value is not None:
                data.append(str(value))

        return ' '.join(data)

    def get_keys(self):
        """Returns all the data of the specified record."""
        return OrderedDict.keys(self)

    def hash(self):
        """Returns the hash value for this object."""
        return hash(frozenset(map(lambda x: str(x).lower(), self.items())))

    def match(self, other):
        """Checks whether the specified record matches to this one."""

        self_keys = self.get_keys()
        other_keys = other.get_keys()
        keys = []
        keys.extend(self_keys)
        keys.extend(other_keys)

        for key in keys:

            if key == self.TIME:
                # TIME field is ignored at comparison.
                continue

            if not ((key in self_keys) and (key in other_keys)):
                return False

            self_item = self.get(key)
            other_item = other.get(key)
            self_none = self_item is None
            other_none = other_item is None

            if self_none != other_none:
                return False

            self_str = str(self_item)
            other_str = str(other_item)
            if (not self_none) and (not other_none) and (self_str.lower() != other_str.lower()):
                return False

        return True

    def __eq__(self, other):
        """Compare record fields in case insensitive mode."""

        self_keys = self.get_keys()
        other_keys = other.get_keys()
        keys = []
        keys.extend(self_keys)
        keys.extend(other_keys)

        for key in keys:

            if not ((key in self_keys) and (key in other_keys)):
                return False

            self_item = self.get(key)
            other_item = other.get(key)
            self_none = self_item is None
            other_none = other_item is None

            if self_none != other_none:
                return False

            self_str = str(self_item)
            other_str = str(other_item)
            if (not self_none) and (not other_none) and (self_str.lower() != other_str.lower()):
                return False

        return True
