"""Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


from traceutils.logger import Logger
from traceutils.oracle import Oracle
from traceutils.trace_parser import TraceParser
import argparse
import logging
import os.path
import sys


class TraceMatcher(argparse.ArgumentParser):
    """Traces matching component."""

    def main(self):
        """Matches traces that are read from command line arguments."""

        self.add_argument('files', metavar='FILE', nargs='+', help='Path to trace file')
        self.add_argument('-w', '--window-size', metavar='W_SIZE', nargs='?', dest='window_size',
                          const=1, default=1, type=int, help='Matching window size in ticks (default: 1)')
        self.add_argument('-fds', '--first-dif-stop', dest='first_diff_stop', action='store_true',
                          help='Stop the tool when the first difference is detected (default: disabled)')
        self.add_argument('-sr', '--skip-rest', dest='skip_rest', action='store_true',
                          help='Skips matching for the rest of the longest trace (default: disabled)')
        self.add_argument('-d', '--debug', dest='debug_level', action='store_true',
                          help='Enables printing for debug messages (default: disabled)')
        args = self.parse_args()

        Logger(logging.INFO if not args.debug_level else logging.DEBUG)

        file_paths = args.files
        if not file_paths:
            self.error("List of input files is empty.")

        file_parsers = list()
        for file_path in file_paths:
            if os.path.isfile(file_path):
                file_parsers.append(TraceParser(file_path))
            else:
                self.error("Can't find file: %s\n" % file_path)

        # Sort parsers by the lengths of their trace files
        file_parsers.sort(key=lambda x: x.get_trace_length(), reverse=False)

        oracle = Oracle(args.window_size, args.first_diff_stop)
        self.register_records(file_parsers, oracle, args.skip_rest)

        oracle.check()

    def register_records(self, file_parsers, oracle, skip_rest):
        """Register all the records are produced by the specified file parsers."""
        while self._has_record(file_parsers):
            if len(file_parsers) == 1:
                only_parser = file_parsers[0]
                logging.debug('The only parser is active for the trace: %s' % only_parser.get_trace_name())
                valuable_record = only_parser.next_valuable_record()
                if valuable_record is None:
                    return
                logging.debug('The valuable record has been found: %s' % valuable_record.to_string())
                only_trace = only_parser.get_trace()
                trace_name = only_trace.name()
                if oracle.future_record_for_other_traces(valuable_record, trace_name):

                    if skip_rest:
                        logging.info('The rest of the trace \'%s\' is skipped starting from:\n %s'
                                     % (trace_name, valuable_record.to_string()))
                        return

                    else:
                        Logger.non_match_rec_hdr()
                        logging.info('\nTrace=%s\n' % trace_name)
                        logging.info(valuable_record.to_string())
                        only_trace.dump_rest_lines()
                        return

            for parser in file_parsers:
                if parser.has_next_record():
                    trace_name = parser.get_trace().name()
                    ret = oracle.register_record(parser.next_record(), trace_name)
                    while ret < 0:
                        if (ret == -2) or (ret == -3):
                            return
                        ret = oracle.register_record(parser.next_record(), trace_name)

                else:
                    file_parsers.remove(parser)

    def error(self, message):
        """Prints error message, then help string, then exits with non-zer value."""
        logging.error(message)
        self.print_help()
        sys.exit(2)

    @staticmethod
    def _has_record(parsers):
        """Checks whether at least one parser from the specified list can provide next record."""
        has_record = False

        for parser in parsers:
            if parser.has_next_record():
                logging.debug('Parser for \'%s\' has next record' % parser.get_trace_name())
                has_record = True
                break

        return has_record

if __name__ == "__main__":
    sys.exit(TraceMatcher().main())
